@@ -26,7 +26,7 @@ import math
 import tempfile
 import urlparse
 
-from glance.common import config
+from glance.common import cfg
 from glance.common import exception
 import glance.store
 import glance.store.base
@@ -38,8 +38,9 @@ except ImportError:
     pass
 
 DEFAULT_CONTAINER = 'glance'
-DEFAULT_LARGE_OBJECT_SIZE = 5 * 1024 * 1024 * 1024  # 5GB
-DEFAULT_LARGE_OBJECT_CHUNK_SIZE = 200 * 1024 * 1024  # 200M
+DEFAULT_LARGE_OBJECT_SIZE = 5 * 1024  # 5GB
+DEFAULT_LARGE_OBJECT_CHUNK_SIZE = 200  # 200M
+ONE_MB = 1000 * 1024
 
 logger = logging.getLogger('glance.store.swift')
 
@@ -79,12 +80,14 @@ class StoreLocation(glance.store.location.StoreLocation):
             authurl = authurl[7:]
         elif authurl.startswith('https://'):
             authurl = authurl[8:]
-        return "%s://%s%s/%s/%s" % (
-            self.scheme,
-            self._get_credstring(),
-            authurl,
-            self.container,
-            self.obj)
+
+        credstring = self._get_credstring()
+        authurl = authurl.strip('/')
+        container = self.container.strip('/')
+        obj = self.obj.strip('/')
+
+        return '%s://%s%s/%s/%s' % (self.scheme, credstring, authurl,
+                                    container, obj)
 
     def parse_uri(self, uri):
         """
@@ -183,7 +186,25 @@ class Store(glance.store.base.Store):
 
     CHUNKSIZE = 65536
 
+    opts = [
+        cfg.BoolOpt('swift_enable_snet', default=False),
+        cfg.StrOpt('swift_store_auth_address'),
+        cfg.StrOpt('swift_store_user'),
+        cfg.StrOpt('swift_store_key'),
+        cfg.StrOpt('swift_store_container',
+                   default=DEFAULT_CONTAINER),
+        cfg.IntOpt('swift_store_large_object_size',
+                   default=DEFAULT_LARGE_OBJECT_SIZE),
+        cfg.IntOpt('swift_store_large_object_chunk_size',
+                   default=DEFAULT_LARGE_OBJECT_CHUNK_SIZE),
+        cfg.BoolOpt('swift_store_create_container_on_put', default=False),
+        ]
+
     def configure(self):
+        self.conf.register_opts(self.opts)
+        self.snet = self.conf.swift_enable_snet
+
+    def configure_add(self):
         """
         Configure the Store to use the stored configuration options
         Any store that needs special configuration should implement
@@ -193,24 +214,17 @@ class Store(glance.store.base.Store):
         self.auth_address = self._option_get('swift_store_auth_address')
         self.user = self._option_get('swift_store_user')
         self.key = self._option_get('swift_store_key')
-        self.container = self.options.get('swift_store_container',
-                                          DEFAULT_CONTAINER)
+        self.container = self.conf.swift_store_container
         try:
-            if self.options.get('swift_store_large_object_size'):
-                self.large_object_size = int(
-                    self.options.get('swift_store_large_object_size')
-                    ) * (1024 * 1024)  # Size specified in MB in conf files
-            else:
-                self.large_object_size = DEFAULT_LARGE_OBJECT_SIZE
-
-            if self.options.get('swift_store_large_object_chunk_size'):
-                self.large_object_chunk_size = int(
-                    self.options.get('swift_store_large_object_chunk_size')
-                    ) * (1024 * 1024)  # Size specified in MB in conf files
-            else:
-                self.large_object_chunk_size = DEFAULT_LARGE_OBJECT_CHUNK_SIZE
-        except Exception, e:
-            reason = _("Error in configuration options: %s") % e
+            # The config file has swift_store_large_object_*size in MB, but
+            # internally we store it in bytes, since the image_size parameter
+            # passed to add() is also in bytes.
+            self.large_object_size = \
+                self.conf.swift_store_large_object_size * ONE_MB
+            self.large_object_chunk_size = \
+                self.conf.swift_store_large_object_chunk_size * ONE_MB
+        except cfg.ConfigFileValueError, e:
+            reason = _("Error in configuration conf: %s") % e
             logger.error(reason)
             raise exception.BadStoreConfiguration(store_name="swift",
                                                   reason=reason)
@@ -224,9 +238,6 @@ class Store(glance.store.base.Store):
         else:  # Defaults https
             self.full_auth_address = 'https://' + self.auth_address
 
-        self.snet = config.get_option(
-            self.options, 'swift_enable_snet', type='bool', default=False)
-
     def get(self, location):
         """
         Takes a `glance.store.location.Location` object that indicates
@@ -269,12 +280,12 @@ class Store(glance.store.base.Store):
         snet = self.snet
         logger.debug(_("Creating Swift connection with "
                      "(auth_address=%(auth_url)s, user=%(user)s, "
-                     "key=%(key)s, snet=%(snet)s)") % locals())
+                     "snet=%(snet)s)") % locals())
         return swift_client.Connection(
             authurl=auth_url, user=user, key=key, snet=snet)
 
     def _option_get(self, param):
-        result = self.options.get(param)
+        result = getattr(self.conf, param)
         if not result:
             reason = (_("Could not find %(param)s in configuration "
                         "options.") % locals())
@@ -314,11 +325,14 @@ class Store(glance.store.base.Store):
               in size. So, if the image is greater than 5GB, we write
               chunks of image data to Swift and then write an manifest
               to Swift that contains information about the chunks.
+              This same chunking process is used by default for images
+              of an unknown size, as pushing them directly to swift would
+              fail if the image turns out to be greater than 5GB.
         """
         swift_conn = self._make_swift_connection(
             auth_url=self.full_auth_address, user=self.user, key=self.key)
 
-        create_container_if_missing(self.container, swift_conn, self.options)
+        create_container_if_missing(self.container, swift_conn, self.conf)
 
         obj_name = str(image_id)
         location = StoreLocation({'scheme': self.scheme,
@@ -331,52 +345,67 @@ class Store(glance.store.base.Store):
         logger.debug(_("Adding image object '%(obj_name)s' "
                        "to Swift") % locals())
         try:
-            if image_size < self.large_object_size:
-                # image_size == 0 is when we don't know the size
-                # of the image. This can occur with older clients
-                # that don't inspect the payload size, and we simply
-                # try to put the object into Swift and hope for the
-                # best...
+            if image_size > 0 and image_size < self.large_object_size:
+                # Image size is known, and is less than large_object_size.
+                # Send to Swift with regular PUT.
                 obj_etag = swift_conn.put_object(self.container, obj_name,
-                                                 image_file)
+                                                 image_file,
+                                                 content_length=image_size)
             else:
-                # Write the image into Swift in chunks. We cannot
-                # stream chunks of the webob.Request.body_file, unfortunately,
-                # so we must write chunks of the body_file into a temporary
-                # disk buffer, and then pass this disk buffer to Swift.
-                bytes_left = image_size
+                # Write the image into Swift in chunks.
                 chunk_id = 1
-                total_chunks = int(math.ceil(
-                    float(image_size) / float(self.large_object_chunk_size)))
+                if image_size > 0:
+                    total_chunks = str(int(
+                        math.ceil(float(image_size) /
+                                  float(self.large_object_chunk_size))))
+                else:
+                    # image_size == 0 is when we don't know the size
+                    # of the image. This can occur with older clients
+                    # that don't inspect the payload size.
+                    logger.debug(_("Cannot determine image size. Adding as a "
+                                   "segmented object to Swift."))
+                    total_chunks = '?'
+
                 checksum = hashlib.md5()
-                while bytes_left > 0:
-                    with tempfile.NamedTemporaryFile() as disk_buffer:
-                        chunk_size = min(self.large_object_chunk_size,
-                                         bytes_left)
-                        logger.debug(_("Writing %(chunk_size)d bytes for "
-                                       "chunk %(chunk_id)d/"
-                                       "%(total_chunks)d to disk buffer "
-                                       "for image %(image_id)s")
-                                     % locals())
-                        chunk = image_file.read(chunk_size)
-                        checksum.update(chunk)
-                        disk_buffer.write(chunk)
-                        disk_buffer.flush()
-                        logger.debug(_("Writing chunk %(chunk_id)d/"
-                                       "%(total_chunks)d to Swift "
-                                       "for image %(image_id)s")
-                                     % locals())
-                        chunk_etag = swift_conn.put_object(
-                            self.container,
-                            "%s-%05d" % (obj_name, chunk_id),
-                            open(disk_buffer.name, 'rb'))
-                        logger.debug(_("Wrote chunk %(chunk_id)d/"
-                                       "%(total_chunks)d to Swift "
-                                       "returning MD5 of content: "
-                                       "%(chunk_etag)s")
-                                     % locals())
-                    bytes_left -= self.large_object_chunk_size
+                combined_chunks_size = 0
+                while True:
+                    chunk_size = self.large_object_chunk_size
+                    if image_size == 0:
+                        content_length = None
+                    else:
+                        left = image_size - combined_chunks_size
+                        if left == 0:
+                            break
+                        if chunk_size > left:
+                            chunk_size = left
+                        content_length = chunk_size
+
+                    chunk_name = "%s-%05d" % (obj_name, chunk_id)
+                    reader = ChunkReader(image_file, checksum, chunk_size)
+                    chunk_etag = swift_conn.put_object(
+                        self.container, chunk_name, reader,
+                        content_length=content_length)
+                    bytes_read = reader.bytes_read
+                    logger.debug(_("Wrote chunk %(chunk_id)d/"
+                                   "%(total_chunks)s of length %(bytes_read)d "
+                                   "to Swift returning MD5 of content: "
+                                   "%(chunk_etag)s")
+                                 % locals())
+
+                    if bytes_read == 0:
+                        # Delete the last chunk, because it's of zero size.
+                        # This will happen if image_size == 0.
+                        logger.debug(_("Deleting final zero-length chunk"))
+                        swift_conn.delete_object(self.container, chunk_name)
+                        break
+
                     chunk_id += 1
+                    combined_chunks_size += bytes_read
+
+                # In the case we have been given an unknown image size,
+                # set the image_size to the total size of the combined chunks.
+                if image_size == 0:
+                    image_size = combined_chunks_size
 
                 # Now we write the object manifest and return the
                 # manifest's etag...
@@ -446,7 +475,9 @@ class Store(glance.store.base.Store):
                     # be hogging up network or file I/O here...
                     swift_conn.delete_object(obj_container, segment['name'])
 
-            swift_conn.delete_object(loc.container, loc.obj)
+            else:
+                swift_conn.delete_object(loc.container, loc.obj)
+
         except swift_client.ClientException, e:
             if e.http_status == httplib.NOT_FOUND:
                 uri = location.get_store_uri()
@@ -456,23 +487,37 @@ class Store(glance.store.base.Store):
                 raise
 
 
-def create_container_if_missing(container, swift_conn, options):
+class ChunkReader(object):
+    def __init__(self, fd, checksum, total):
+        self.fd = fd
+        self.checksum = checksum
+        self.total = total
+        self.bytes_read = 0
+
+    def read(self, i):
+        left = self.total - self.bytes_read
+        if i > left:
+            i = left
+        result = self.fd.read(i)
+        self.bytes_read += len(result)
+        self.checksum.update(result)
+        return result
+
+
+def create_container_if_missing(container, swift_conn, conf):
     """
     Creates a missing container in Swift if the
     ``swift_store_create_container_on_put`` option is set.
 
     :param container: Name of container to create
     :param swift_conn: Connection to Swift
-    :param options: Option mapping
+    :param conf: Option mapping
     """
     try:
         swift_conn.head_container(container)
     except swift_client.ClientException, e:
         if e.http_status == httplib.NOT_FOUND:
-            add_container = config.get_option(options,
-                                'swift_store_create_container_on_put',
-                                type='bool', default=False)
-            if add_container:
+            if conf.swift_store_create_container_on_put:
                 try:
                     swift_conn.put_container(container)
                 except ClientException, e:
