@@ -17,12 +17,14 @@
 
 """Functional test case that utilizes httplib2 against the API server"""
 
+import datetime
 import hashlib
 import httplib2
 import json
 import os
 import tempfile
 
+from glance.common import utils
 from glance.tests import functional
 from glance.tests.utils import execute, skip_if_disabled
 
@@ -43,26 +45,24 @@ class TestApi(functional.FunctionalTest):
         - Verify no public images
         1. GET /images/detail
         - Verify no public images
-        2. HEAD /images/1
-        - Verify 404 returned
-        3. POST /images with public image named Image1
+        2. POST /images with public image named Image1
         and no custom properties
         - Verify 201 returned
-        4. HEAD /images/1
+        3. HEAD image
         - Verify HTTP headers have correct information we just added
-        5. GET /images/1
+        4. GET image
         - Verify all information on image we just added is correct
-        6. GET /images
+        5. GET /images
         - Verify the image we just added is returned
-        7. GET /images/detail
+        6. GET /images/detail
         - Verify the image we just added is returned
-        8. PUT /images/1 with custom properties of "distro" and "arch"
+        7. PUT image with custom properties of "distro" and "arch"
         - Verify 200 returned
-        9. GET /images/1
+        8. GET image
         - Verify updated information about image was stored
-        10. PUT /images/1
+        9. PUT image
         - Remove a previously existing property.
-        11. PUT /images/1
+        10. PUT image
         - Add a previously deleted property.
         """
         self.cleanup()
@@ -84,14 +84,7 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(response.status, 200)
         self.assertEqual(content, '{"images": []}')
 
-        # 2. HEAD /images/1
-        # Verify 404 returned
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
-        http = httplib2.Http()
-        response, content = http.request(path, 'HEAD')
-        self.assertEqual(response.status, 404)
-
-        # 3. POST /images with public image named Image1
+        # 2. POST /images with public image named Image1
         # attribute and no custom properties. Verify a 200 OK is returned
         image_data = "*" * FIVE_KB
         headers = {'Content-Type': 'application/octet-stream',
@@ -103,29 +96,32 @@ class TestApi(functional.FunctionalTest):
                                          body=image_data)
         self.assertEqual(response.status, 201)
         data = json.loads(content)
+        image_id = data['image']['id']
         self.assertEqual(data['image']['checksum'],
                          hashlib.md5(image_data).hexdigest())
         self.assertEqual(data['image']['size'], FIVE_KB)
         self.assertEqual(data['image']['name'], "Image1")
         self.assertEqual(data['image']['is_public'], True)
 
-        # 4. HEAD /images/1
+        # 3. HEAD image
         # Verify image found now
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
         http = httplib2.Http()
         response, content = http.request(path, 'HEAD')
         self.assertEqual(response.status, 200)
         self.assertEqual(response['x-image-meta-name'], "Image1")
 
-        # 5. GET /images/1
+        # 4. GET image
         # Verify all information on image we just added is correct
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
         http = httplib2.Http()
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
         expected_image_headers = {
-            'x-image-meta-id': '1',
+            'x-image-meta-id': image_id,
             'x-image-meta-name': 'Image1',
             'x-image-meta-is_public': 'True',
             'x-image-meta-status': 'active',
@@ -154,7 +150,7 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(hashlib.md5(content).hexdigest(),
                          hashlib.md5("*" * FIVE_KB).hexdigest())
 
-        # 6. GET /images
+        # 5. GET /images
         # Verify no public images
         path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
         http = httplib2.Http()
@@ -164,13 +160,13 @@ class TestApi(functional.FunctionalTest):
         expected_result = {"images": [
             {"container_format": None,
              "disk_format": None,
-             "id": 1,
+             "id": image_id,
              "name": "Image1",
              "checksum": "c2e5db72bd7fd153f53ede5da5a06de3",
              "size": 5120}]}
         self.assertEqual(json.loads(content), expected_result)
 
-        # 7. GET /images/detail
+        # 6. GET /images/detail
         # Verify image and all its metadata
         path = "http://%s:%d/v1/images/detail" % ("0.0.0.0", self.api_port)
         http = httplib2.Http()
@@ -183,7 +179,7 @@ class TestApi(functional.FunctionalTest):
             "deleted": False,
             "container_format": None,
             "disk_format": None,
-            "id": 1,
+            "id": image_id,
             "is_public": True,
             "deleted_at": None,
             "properties": {},
@@ -198,11 +194,12 @@ class TestApi(functional.FunctionalTest):
                                expected_value,
                                image['images'][0][expected_key]))
 
-        # 8. PUT /images/1 with custom properties of "distro" and "arch"
+        # 7. PUT image with custom properties of "distro" and "arch"
         # Verify 200 returned
         headers = {'X-Image-Meta-Property-Distro': 'Ubuntu',
                    'X-Image-Meta-Property-Arch': 'x86_64'}
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
         http = httplib2.Http()
         response, content = http.request(path, 'PUT', headers=headers)
         self.assertEqual(response.status, 200)
@@ -210,7 +207,7 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(data['image']['properties']['arch'], "x86_64")
         self.assertEqual(data['image']['properties']['distro'], "Ubuntu")
 
-        # 9. GET /images/detail
+        # 8. GET /images/detail
         # Verify image and all its metadata
         path = "http://%s:%d/v1/images/detail" % ("0.0.0.0", self.api_port)
         http = httplib2.Http()
@@ -223,7 +220,7 @@ class TestApi(functional.FunctionalTest):
             "deleted": False,
             "container_format": None,
             "disk_format": None,
-            "id": 1,
+            "id": image_id,
             "is_public": True,
             "deleted_at": None,
             "properties": {'distro': 'Ubuntu', 'arch': 'x86_64'},
@@ -238,9 +235,10 @@ class TestApi(functional.FunctionalTest):
                                expected_value,
                                image['images'][0][expected_key]))
 
-        # 10. PUT /images/1 and remove a previously existing property.
+        # 9. PUT image and remove a previously existing property.
         headers = {'X-Image-Meta-Property-Arch': 'x86_64'}
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
         http = httplib2.Http()
         response, content = http.request(path, 'PUT', headers=headers)
         self.assertEqual(response.status, 200)
@@ -252,10 +250,11 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(len(data['properties']), 1)
         self.assertEqual(data['properties']['arch'], "x86_64")
 
-        # 11. PUT /images/1 and add a previously deleted property.
+        # 10. PUT image and add a previously deleted property.
         headers = {'X-Image-Meta-Property-Distro': 'Ubuntu',
                    'X-Image-Meta-Property-Arch': 'x86_64'}
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
         http = httplib2.Http()
         response, content = http.request(path, 'PUT', headers=headers)
         self.assertEqual(response.status, 200)
@@ -269,6 +268,13 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(data['properties']['arch'], "x86_64")
         self.assertEqual(data['properties']['distro'], "Ubuntu")
 
+        # DELETE image
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
+        http = httplib2.Http()
+        response, content = http.request(path, 'DELETE')
+        self.assertEqual(response.status, 200)
+
         self.stop_servers()
 
     @skip_if_disabled
@@ -286,11 +292,11 @@ class TestApi(functional.FunctionalTest):
         - Verify 201 returned
         2. GET /images
         - Verify one public image
-        3. HEAD /images/1
+        3. HEAD image
         - Verify image now in queued status
-        4. PUT /images/1 with image data
+        4. PUT image with image data
         - Verify 200 returned
-        5. HEAD /images/1
+        5. HEAD images
         - Verify image now in active status
         6. GET /images
         - Verify one public image
@@ -324,6 +330,8 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(data['image']['name'], "Image1")
         self.assertEqual(data['image']['is_public'], True)
 
+        image_id = data['image']['id']
+
         # 2. GET /images
         # Verify 1 public image
         path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
@@ -331,7 +339,7 @@ class TestApi(functional.FunctionalTest):
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
         data = json.loads(content)
-        self.assertEqual(data['images'][0]['id'], 1)
+        self.assertEqual(data['images'][0]['id'], image_id)
         self.assertEqual(data['images'][0]['checksum'], None)
         self.assertEqual(data['images'][0]['size'], 0)
         self.assertEqual(data['images'][0]['container_format'], None)
@@ -340,19 +348,21 @@ class TestApi(functional.FunctionalTest):
 
         # 3. HEAD /images
         # Verify status is in queued
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
         http = httplib2.Http()
         response, content = http.request(path, 'HEAD')
         self.assertEqual(response.status, 200)
         self.assertEqual(response['x-image-meta-name'], "Image1")
         self.assertEqual(response['x-image-meta-status'], "queued")
         self.assertEqual(response['x-image-meta-size'], '0')
-        self.assertEqual(response['x-image-meta-id'], '1')
+        self.assertEqual(response['x-image-meta-id'], image_id)
 
-        # 4. PUT /images/1 with image data, verify 200 returned
+        # 4. PUT image with image data, verify 200 returned
         image_data = "*" * FIVE_KB
         headers = {'Content-Type': 'application/octet-stream'}
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                             image_id)
         http = httplib2.Http()
         response, content = http.request(path, 'PUT', headers=headers,
                                          body=image_data)
@@ -366,7 +376,8 @@ class TestApi(functional.FunctionalTest):
 
         # 5. HEAD /images
         # Verify status is in active
-        path = "http://%s:%d/v1/images/1" % ("0.0.0.0", self.api_port)
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
         http = httplib2.Http()
         response, content = http.request(path, 'HEAD')
         self.assertEqual(response.status, 200)
@@ -382,12 +393,19 @@ class TestApi(functional.FunctionalTest):
         data = json.loads(content)
         self.assertEqual(data['images'][0]['checksum'],
                          hashlib.md5(image_data).hexdigest())
-        self.assertEqual(data['images'][0]['id'], 1)
+        self.assertEqual(data['images'][0]['id'], image_id)
         self.assertEqual(data['images'][0]['size'], FIVE_KB)
         self.assertEqual(data['images'][0]['container_format'], None)
         self.assertEqual(data['images'][0]['disk_format'], None)
         self.assertEqual(data['images'][0]['name'], "Image1")
 
+        # DELETE image
+        path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                              image_id)
+        http = httplib2.Http()
+        response, content = http.request(path, 'DELETE')
+        self.assertEqual(response.status, 200)
+
         self.stop_servers()
 
     @skip_if_disabled
@@ -638,6 +656,8 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(response.status, 200)
         self.assertEqual(content, '{"images": []}')
 
+        image_ids = []
+
         # 1. POST /images with three public images, and one private image
         # with various attributes
         headers = {'Content-Type': 'application/octet-stream',
@@ -647,6 +667,7 @@ class TestApi(functional.FunctionalTest):
                    'X-Image-Meta-Disk-Format': 'vdi',
                    'X-Image-Meta-Size': '19',
                    'X-Image-Meta-Is-Public': 'True',
+                   'X-Image-Meta-Protected': 'True',
                    'X-Image-Meta-Property-pants': 'are on'}
         path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
         http = httplib2.Http()
@@ -655,6 +676,7 @@ class TestApi(functional.FunctionalTest):
         data = json.loads(content)
         self.assertEqual(data['image']['properties']['pants'], "are on")
         self.assertEqual(data['image']['is_public'], True)
+        image_ids.append(data['image']['id'])
 
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'My Image!',
@@ -663,6 +685,7 @@ class TestApi(functional.FunctionalTest):
                    'X-Image-Meta-Disk-Format': 'vhd',
                    'X-Image-Meta-Size': '20',
                    'X-Image-Meta-Is-Public': 'True',
+                   'X-Image-Meta-Protected': 'False',
                    'X-Image-Meta-Property-pants': 'are on'}
         path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
         http = httplib2.Http()
@@ -671,6 +694,7 @@ class TestApi(functional.FunctionalTest):
         data = json.loads(content)
         self.assertEqual(data['image']['properties']['pants'], "are on")
         self.assertEqual(data['image']['is_public'], True)
+        image_ids.append(data['image']['id'])
 
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'My Image!',
@@ -679,6 +703,7 @@ class TestApi(functional.FunctionalTest):
                    'X-Image-Meta-Disk-Format': 'ami',
                    'X-Image-Meta-Size': '21',
                    'X-Image-Meta-Is-Public': 'True',
+                   'X-Image-Meta-Protected': 'False',
                    'X-Image-Meta-Property-pants': 'are off'}
         path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
         http = httplib2.Http()
@@ -687,6 +712,7 @@ class TestApi(functional.FunctionalTest):
         data = json.loads(content)
         self.assertEqual(data['image']['properties']['pants'], "are off")
         self.assertEqual(data['image']['is_public'], True)
+        image_ids.append(data['image']['id'])
 
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'My Private Image',
@@ -694,13 +720,15 @@ class TestApi(functional.FunctionalTest):
                    'X-Image-Meta-Container-Format': 'ami',
                    'X-Image-Meta-Disk-Format': 'ami',
                    'X-Image-Meta-Size': '22',
-                   'X-Image-Meta-Is-Public': 'False'}
+                   'X-Image-Meta-Is-Public': 'False',
+                   'X-Image-Meta-Protected': 'False'}
         path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
         data = json.loads(content)
         self.assertEqual(data['image']['is_public'], False)
+        image_ids.append(data['image']['id'])
 
         # 2. GET /images
         # Verify three public images
@@ -827,7 +855,31 @@ class TestApi(functional.FunctionalTest):
         for image in data['images']:
             self.assertNotEqual(image['name'], "My Private Image")
 
-        # 12. GET /images with property filter
+        # 12. Get /images with protected=False filter
+        # Verify correct images returned with property
+        params = "protected=False"
+        path = "http://%s:%d/v1/images?%s" % (
+               "0.0.0.0", self.api_port, params)
+        response, content = http.request(path, 'GET')
+        self.assertEqual(response.status, 200)
+        data = json.loads(content)
+        self.assertEqual(len(data['images']), 2)
+        for image in data['images']:
+            self.assertNotEqual(image['name'], "Image1")
+
+        # 13. Get /images with protected=True filter
+        # Verify correct images returned with property
+        params = "protected=True"
+        path = "http://%s:%d/v1/images?%s" % (
+               "0.0.0.0", self.api_port, params)
+        response, content = http.request(path, 'GET')
+        self.assertEqual(response.status, 200)
+        data = json.loads(content)
+        self.assertEqual(len(data['images']), 1)
+        for image in data['images']:
+            self.assertEqual(image['name'], "Image1")
+
+        # 14. GET /images with property filter
         # Verify correct images returned with property
         params = "property-pants=are%20on"
         path = "http://%s:%d/v1/images/detail?%s" % (
@@ -839,7 +891,7 @@ class TestApi(functional.FunctionalTest):
         for image in data['images']:
             self.assertEqual(image['properties']['pants'], "are on")
 
-        # 13. GET /images with property filter and name filter
+        # 15. GET /images with property filter and name filter
         # Verify correct images returned with property and name
         # Make sure you quote the url when using more than one param!
         params = "name=My%20Image!&property-pants=are%20on"
@@ -853,6 +905,26 @@ class TestApi(functional.FunctionalTest):
             self.assertEqual(image['properties']['pants'], "are on")
             self.assertEqual(image['name'], "My Image!")
 
+        # 16. GET /images with past changes-since filter
+        dt1 = datetime.datetime.utcnow() - datetime.timedelta(1)
+        iso1 = utils.isotime(dt1)
+        params = "changes-since=%s" % iso1
+        path = "http://%s:%d/v1/images?%s" % ("0.0.0.0", self.api_port, params)
+        response, content = http.request(path, 'GET')
+        self.assertEqual(response.status, 200)
+        data = json.loads(content)
+        self.assertEqual(len(data['images']), 3)
+
+        # 17. GET /images with future changes-since filter
+        dt2 = datetime.datetime.utcnow() + datetime.timedelta(1)
+        iso2 = utils.isotime(dt2)
+        params = "changes-since=%s" % iso2
+        path = "http://%s:%d/v1/images?%s" % ("0.0.0.0", self.api_port, params)
+        response, content = http.request(path, 'GET')
+        self.assertEqual(response.status, 200)
+        data = json.loads(content)
+        self.assertEqual(len(data['images']), 0)
+
         self.stop_servers()
 
     @skip_if_disabled
@@ -871,6 +943,8 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(response.status, 200)
         self.assertEqual(content, '{"images": []}')
 
+        image_ids = []
+
         # 1. POST /images with three public images with various attributes
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'Image1',
@@ -879,6 +953,7 @@ class TestApi(functional.FunctionalTest):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
+        image_ids.append(json.loads(content)['image']['id'])
 
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'Image2',
@@ -887,6 +962,7 @@ class TestApi(functional.FunctionalTest):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
+        image_ids.append(json.loads(content)['image']['id'])
 
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'Image3',
@@ -895,52 +971,69 @@ class TestApi(functional.FunctionalTest):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
+        image_ids.append(json.loads(content)['image']['id'])
 
-        # 2. GET /images with limit of 2
+        # 2. GET /images with all images
+        path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
+        response, content = http.request(path, 'GET')
+        self.assertEqual(response.status, 200)
+        images = json.loads(content)['images']
+        self.assertEqual(len(images), 3)
+        print images
+
+        # 3. GET /images with limit of 2
         # Verify only two images were returned
         params = "limit=2"
         path = "http://%s:%d/v1/images?%s" % (
                "0.0.0.0", self.api_port, params)
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
-        self.assertEqual(len(data['images']), 2)
-        self.assertEqual(data['images'][0]['id'], 3)
-        self.assertEqual(data['images'][1]['id'], 2)
+        data = json.loads(content)['images']
+        self.assertEqual(len(data), 2)
+        self.assertEqual(data[0]['id'], images[0]['id'])
+        self.assertEqual(data[1]['id'], images[1]['id'])
 
-        # 3. GET /images with marker
+        # 4. GET /images with marker
         # Verify only two images were returned
-        params = "marker=3"
+        params = "marker=%s" % images[0]['id']
         path = "http://%s:%d/v1/images?%s" % (
                "0.0.0.0", self.api_port, params)
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
-        self.assertEqual(len(data['images']), 2)
-        self.assertEqual(data['images'][0]['id'], 2)
-        self.assertEqual(data['images'][1]['id'], 1)
+        data = json.loads(content)['images']
+        self.assertEqual(len(data), 2)
+        self.assertEqual(data[0]['id'], images[1]['id'])
+        self.assertEqual(data[1]['id'], images[2]['id'])
 
-        # 4. GET /images with marker and limit
+        # 5. GET /images with marker and limit
         # Verify only one image was returned with the correct id
-        params = "limit=1&marker=2"
+        params = "limit=1&marker=%s" % images[1]['id']
         path = "http://%s:%d/v1/images?%s" % (
                "0.0.0.0", self.api_port, params)
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
-        self.assertEqual(len(data['images']), 1)
-        self.assertEqual(data['images'][0]['id'], 1)
+        data = json.loads(content)['images']
+        self.assertEqual(len(data), 1)
+        self.assertEqual(data[0]['id'], images[2]['id'])
 
-        # 5. GET /images/detail with marker and limit
+        # 6. GET /images/detail with marker and limit
         # Verify only one image was returned with the correct id
-        params = "limit=1&marker=3"
+        params = "limit=1&marker=%s" % images[1]['id']
         path = "http://%s:%d/v1/images?%s" % (
                "0.0.0.0", self.api_port, params)
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
-        self.assertEqual(len(data['images']), 1)
-        self.assertEqual(data['images'][0]['id'], 2)
+        data = json.loads(content)['images']
+        self.assertEqual(len(data), 1)
+        self.assertEqual(data[0]['id'], images[2]['id'])
+
+        # DELETE images
+        for image_id in image_ids:
+            path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                                  image_id)
+            http = httplib2.Http()
+            response, content = http.request(path, 'DELETE')
+            self.assertEqual(response.status, 200)
 
         self.stop_servers()
 
@@ -961,6 +1054,7 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(content, '{"images": []}')
 
         # 1. POST /images with three public images with various attributes
+        image_ids = []
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'Image1',
                    'X-Image-Meta-Status': 'active',
@@ -972,6 +1066,7 @@ class TestApi(functional.FunctionalTest):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
+        image_ids.append(json.loads(content)['image']['id'])
 
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'ASDF',
@@ -984,6 +1079,7 @@ class TestApi(functional.FunctionalTest):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
+        image_ids.append(json.loads(content)['image']['id'])
 
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'XYZ',
@@ -996,6 +1092,7 @@ class TestApi(functional.FunctionalTest):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
+        image_ids.append(json.loads(content)['image']['id'])
 
         # 2. GET /images with no query params
         # Verify three public images sorted by created_at desc
@@ -1005,9 +1102,9 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(response.status, 200)
         data = json.loads(content)
         self.assertEqual(len(data['images']), 3)
-        self.assertEqual(data['images'][0]['id'], 3)
-        self.assertEqual(data['images'][1]['id'], 2)
-        self.assertEqual(data['images'][2]['id'], 1)
+        self.assertEqual(data['images'][0]['id'], image_ids[2])
+        self.assertEqual(data['images'][1]['id'], image_ids[1])
+        self.assertEqual(data['images'][2]['id'], image_ids[0])
 
         # 3. GET /images sorted by name asc
         params = 'sort_key=name&sort_dir=asc'
@@ -1017,9 +1114,9 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(response.status, 200)
         data = json.loads(content)
         self.assertEqual(len(data['images']), 3)
-        self.assertEqual(data['images'][0]['id'], 2)
-        self.assertEqual(data['images'][1]['id'], 1)
-        self.assertEqual(data['images'][2]['id'], 3)
+        self.assertEqual(data['images'][0]['id'], image_ids[1])
+        self.assertEqual(data['images'][1]['id'], image_ids[0])
+        self.assertEqual(data['images'][2]['id'], image_ids[2])
 
         # 4. GET /images sorted by size desc
         params = 'sort_key=size&sort_dir=desc'
@@ -1029,23 +1126,23 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(response.status, 200)
         data = json.loads(content)
         self.assertEqual(len(data['images']), 3)
-        self.assertEqual(data['images'][0]['id'], 1)
-        self.assertEqual(data['images'][1]['id'], 3)
-        self.assertEqual(data['images'][2]['id'], 2)
+        self.assertEqual(data['images'][0]['id'], image_ids[0])
+        self.assertEqual(data['images'][1]['id'], image_ids[2])
+        self.assertEqual(data['images'][2]['id'], image_ids[1])
 
         # 5. GET /images sorted by size desc with a marker
-        params = 'sort_key=size&sort_dir=desc&marker=1'
+        params = 'sort_key=size&sort_dir=desc&marker=%s' % image_ids[0]
         path = "http://%s:%d/v1/images?%s" % ("0.0.0.0", self.api_port, params)
         http = httplib2.Http()
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
         data = json.loads(content)
         self.assertEqual(len(data['images']), 2)
-        self.assertEqual(data['images'][0]['id'], 3)
-        self.assertEqual(data['images'][1]['id'], 2)
+        self.assertEqual(data['images'][0]['id'], image_ids[2])
+        self.assertEqual(data['images'][1]['id'], image_ids[1])
 
         # 6. GET /images sorted by name asc with a marker
-        params = 'sort_key=name&sort_dir=asc&marker=3'
+        params = 'sort_key=name&sort_dir=asc&marker=%s' % image_ids[2]
         path = "http://%s:%d/v1/images?%s" % ("0.0.0.0", self.api_port, params)
         http = httplib2.Http()
         response, content = http.request(path, 'GET')
@@ -1053,6 +1150,14 @@ class TestApi(functional.FunctionalTest):
         data = json.loads(content)
         self.assertEqual(len(data['images']), 0)
 
+        # DELETE images
+        for image_id in image_ids:
+            path = "http://%s:%d/v1/images/%s" % ("0.0.0.0", self.api_port,
+                                                  image_id)
+            http = httplib2.Http()
+            response, content = http.request(path, 'DELETE')
+            self.assertEqual(response.status, 200)
+
         self.stop_servers()
 
     @skip_if_disabled
@@ -1084,6 +1189,8 @@ class TestApi(functional.FunctionalTest):
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
 
+        image = json.loads(content)['image']
+
         # 2. POST /images with public image named Image1, and ID: 1
         headers = {'Content-Type': 'application/octet-stream',
                    'X-Image-Meta-Name': 'Image1 Update',
@@ -1091,15 +1198,12 @@ class TestApi(functional.FunctionalTest):
                    'X-Image-Meta-Container-Format': 'ovf',
                    'X-Image-Meta-Disk-Format': 'vdi',
                    'X-Image-Meta-Size': '19',
-                   'X-Image-Meta-Id': '1',
+                   'X-Image-Meta-Id': image['id'],
                    'X-Image-Meta-Is-Public': 'True'}
         path = "http://%s:%d/v1/images" % ("0.0.0.0", self.api_port)
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 409)
-        expected = "An image with identifier 1 already exists"
-        self.assertTrue(expected in content,
-                        "Could not find '%s' in '%s'" % (expected, content))
 
         self.stop_servers()
 
@@ -1135,3 +1239,17 @@ class TestApi(functional.FunctionalTest):
         self.assertEqual(response.status, 404)
 
         self.stop_servers()
+
+    @skip_if_disabled
+    def test_unsupported_default_store(self):
+        """
+        We test that a mis-configured default_store causes the API server
+        to fail to start.
+        """
+        self.cleanup()
+        self.api_server.default_store = 'shouldnotexist'
+
+        # ensure that the API server fails to launch
+        self.start_server(self.api_server,
+                          expect_launch=False,
+                          **self.__dict__.copy())
